@@ -192,7 +192,7 @@ impl WebsocatConfiguration2 {
                 }
             }
             (IsItself, IsItself) => {
-                info!("Special mode, expection from usual one-stdio rule. Acting like `cat(1)`");
+                info!("Special mode, exception from usual one-stdio rule. Acting like `cat(1)`");
                 self.s2 = SpecifierStack::from_str("mirror:")?;
                 if self.opts.unidirectional ^ self.opts.unidirectional_reverse {
                     self.opts.unidirectional = false;
@@ -209,11 +209,11 @@ impl WebsocatConfiguration2 {
         {
             if r#async {
                 if self.s1.addrtype.cls.get_name() == "StdioClass" {
-                    debug!("Substituding StdioClass with AsyncStdioClass at the left");
+                    debug!("Substituting StdioClass with AsyncStdioClass at the left");
                     self.s1.addrtype = SpecifierNode{cls:Rc::new(crate::stdio_peer::AsyncStdioClass)};
                 } 
                 if self.s2.addrtype.cls.get_name() == "StdioClass" {
-                    debug!("Substituding StdioClass with AsyncStdioClass at the right");
+                    debug!("Substituting StdioClass with AsyncStdioClass at the right");
                     self.s2.addrtype = SpecifierNode{cls:Rc::new(crate::stdio_peer::AsyncStdioClass)};
                 } 
             }
@@ -471,12 +471,15 @@ impl WebsocatConfiguration2 {
 
     #[cfg(feature = "ssl")]
     fn l_ssl(&mut self, _on_warning: &OnWarning) -> Result<()> {
-        if self.contains_class("TlsAcceptClass") ^ self.opts.pkcs12_der.is_some() {
-            Err("SSL listerer and --pkcs12-der option should go together")?;
+        if self.opts.pkcs12_der.is_some() &&  !self.contains_class("TlsAcceptClass") {
+            Err("--pkcs12-der makes no sense without an TLS connections acceptor")?;
+        }
+        if self.opts.client_pkcs12_der.is_some() && !self.contains_class("WsClientSecureClass") && !self.contains_class("TlsConnectClass") {
+            Err("--client-pkcs12-der makes no sense without wss:// or ssl: connectors")?;
         }
         #[cfg(target_os = "macos")]
         {
-            if self.opts.pkcs12_der.is_some() && self.opts.pkcs12_passwd.is_none() {
+            if (self.opts.pkcs12_der.is_some() && self.opts.pkcs12_passwd.is_none()) || (self.opts.client_pkcs12_der.is_some() && self.opts.client_pkcs12_passwd.is_none()) {
                 _on_warning("PKCS12 archives without password may be unsupported on Mac");
 
                 for x in ::std::env::args() {
@@ -508,7 +511,7 @@ impl WebsocatConfiguration2 {
             if self.opts.unidirectional_reverse || self.opts.exit_on_eof {
                 // OK
             } else {
-                _on_warning("--ping-interval is currenty not very effective without -E or -U")
+                _on_warning("--ping-interval is currently not very effective without -E or -U")
             }
         }
         if self.opts.print_ping_rtts && self.opts.ws_ping_interval.is_none() {
@@ -602,6 +605,37 @@ impl WebsocatConfiguration2 {
         }
         Ok(())
     }
+    fn l_sizelimits(&mut self, _on_warning: &OnWarning) -> Result<()> {
+        if self.opts.max_ws_message_length < self.opts.max_ws_frame_length {
+            _on_warning("Lowering --max-ws-message-length without also lowering --max-ws-frame-length may be meaningless, as the former only affects whether to begin accept a new frame or not, given accumulated message size. Succesfully accepted frames within the frame size limit may exceed the message size.")
+        }
+        Ok(())
+    }
+    fn l_compress(&mut self, _on_warning: &OnWarning) -> Result<()> {
+        let mut cn = 0;
+        let mut un = 0;
+        if self.opts.compress_deflate { cn += 1 }
+        if self.opts.compress_gzip { cn += 1 }
+        if self.opts.compress_zlib { cn += 1 }
+        if self.opts.uncompress_deflate { un += 1 }
+        if self.opts.uncompress_gzip { un += 1 }
+        if self.opts.uncompress_zlib { un += 1 }
+        if cn > 1 {
+            return Err("Multiple --compress-* options specifed")?;
+        }
+        if un > 1 {
+            return Err("Multiple --uncompress-* options specifed")?;
+        }
+
+        #[cfg(not(feature="compression"))]
+        {
+            if cn > 0 || un > 0 {
+                return Err("Compression support is not selected during Websocat compiltion")?;
+            }
+        }
+        Ok(())
+    }
+
 
 
     pub fn lint_and_fixup(&mut self, on_warning: OnWarning) -> Result<()> {
@@ -626,6 +660,8 @@ impl WebsocatConfiguration2 {
         self.l_eeof_unidir(&on_warning)?;
         self.l_udp(&on_warning)?;
         self.l_crypto(&on_warning)?;
+        self.l_sizelimits(&on_warning)?;
+        self.l_compress(&on_warning)?;
 
         // TODO: UDP connect oneshot mode
         // TODO: tests for the linter
