@@ -391,6 +391,21 @@ struct Opt {
     )]
     pkcs12_passwd: Option<String>,
 
+    #[cfg(feature = "ssl")]
+    #[structopt(
+        long = "client-pkcs12-der",
+        help = "[A] Client identity TLS certificate",
+        parse(try_from_os_str = "websocat::ssl_peer::interpret_pkcs12")
+    )]
+    client_pkcs12_der: Option<Vec<u8>>,
+
+    #[cfg(feature = "ssl")]
+    #[structopt(
+        long = "client-pkcs12-passwd",
+        help = "[A] Password for --client-pkcs12-der pkcs12 archive. Required on Mac."
+    )]
+    client_pkcs12_passwd: Option<String>,
+
     #[cfg(feature = "ssl")]
     #[structopt(
         long = "insecure",
@@ -539,6 +554,46 @@ struct Opt {
     /// [A] Override the byte which byte_to_exit_on: overlay looks for
     #[structopt(long = "byte-to-exit-on", default_value = "28")]
     byte_to_exit_on: u8,
+
+    /// [A] Maximum size of incoming WebSocket messages (sans of one data frame), to prevent memory overflow
+    #[structopt(long = "max-ws-message-length", default_value = "209715200")]
+    pub max_ws_message_length: usize,
+    /// [A] Maximum size of incoming WebSocket frames, to prevent memory overflow
+    #[structopt(long = "max-ws-frame-length", default_value = "104857600")]
+    pub max_ws_frame_length: usize,
+
+    /// Prepend copied data with a specified string. Can be specified multiple times.
+    #[structopt(long = "preamble", short="p")]
+    pub preamble: Vec<String>,
+
+    /// Prepend copied data with a specified string (reverse direction). Can be specified multiple times.
+    #[structopt(long = "preamble-reverse", short="P")]
+    pub preamble_reverse: Vec<String>,
+
+
+    /// [A] Compress data coming to a WebSocket using deflate method. Affects only binary WebSocket messages.
+    #[structopt(long = "compress-deflate")]
+    pub compress_deflate: bool,
+
+    /// [A] Compress data coming to a WebSocket using zlib method. Affects only binary WebSocket messages.
+    #[structopt(long = "compress-zlib")]
+    pub compress_zlib: bool,
+
+    /// [A] Compress data coming to a WebSocket using gzip method. Affects only binary WebSocket messages.
+    #[structopt(long = "compress-gzip")]
+    pub compress_gzip: bool,
+
+    /// [A] Uncompress data coming from a WebSocket using deflate method. Affects only binary WebSocket messages.
+    #[structopt(long = "uncompress-deflate")]
+    pub uncompress_deflate: bool,
+
+    /// [A] Uncompress data coming from a WebSocket using deflate method. Affects only binary WebSocket messages.
+    #[structopt(long = "uncompress-zlib")]
+    pub uncompress_zlib: bool,
+
+    /// [A] Uncompress data coming from a WebSocket using deflate method. Affects only binary WebSocket messages.
+    #[structopt(long = "uncompress-gzip")]
+    pub uncompress_gzip: bool,
 }
 
 // TODO: make it byte-oriented/OsStr?
@@ -805,12 +860,24 @@ fn run() -> Result<()> {
             timestamp_monotonic
             print_ping_rtts
             byte_to_exit_on
+            max_ws_message_length
+            max_ws_frame_length
+            preamble
+            preamble_reverse
+            compress_deflate
+            compress_zlib
+            compress_gzip
+            uncompress_deflate
+            uncompress_zlib
+            uncompress_gzip
         );
         #[cfg(feature = "ssl")]
         {
             opts! {
                 pkcs12_der
                 pkcs12_passwd
+                client_pkcs12_der
+                client_pkcs12_passwd
                 tls_insecure
             }
         }
@@ -838,8 +905,12 @@ fn run() -> Result<()> {
 
     let (s1, s2): (String, String) = match (cmd.addr1, cmd.addr2) {
         (None, None) => {
-            help::shorthelp();
-            return Err("No URL specified")?;
+            for x in std::env::args() {
+                if x == "-p" || x == "-P" || x == "--preamble" || x == "--preamble-reverse" {
+                    eprintln!("Warning: all dashless arguments after -p or -P are considered part of the preamble. You may want to move -p/-P to the end of the command line.")
+                }
+            }
+            return Err("No URL specified. Use `websocat --help` to show the help message.")?;
         }
         (Some(cmds1), Some(cmds2)) => {
             // Advanced mode
