@@ -71,6 +71,10 @@ pub struct Options {
     pub pkcs12_der: Option<Vec<u8>>,
     #[derivative(Debug = "ignore")]
     pub pkcs12_passwd: Option<String>,
+    #[derivative(Debug = "ignore")]
+    pub client_pkcs12_der: Option<Vec<u8>>,
+    #[derivative(Debug = "ignore")]
+    pub client_pkcs12_passwd: Option<String>,
     pub tls_insecure: bool,
 
     pub headers_to_env: Vec<String>,
@@ -110,4 +114,19 @@ pub struct Options {
 
     #[default = 0x1c]
     pub byte_to_exit_on: u8,
+
+    #[default = 209715200]
+    pub max_ws_message_length: usize,
+    #[default = 104857600]
+    pub max_ws_frame_length: usize,
+
+    pub preamble: Vec<String>,
+    pub preamble_reverse: Vec<String>,
+
+    pub compress_deflate: bool,
+    pub compress_zlib: bool,
+    pub compress_gzip: bool,
+    pub uncompress_deflate: bool,
+    pub uncompress_zlib: bool,
+    pub uncompress_gzip: bool,
 }
